%election_cups
% Model of sinking time for Election Cups.
%
% LAST UPDATED by Andy French February 2022

function election_cups

%% Inputs %%

%Measured data (with a bit of detergent to reduce surface tension)
d_data = [2,2.5,3,3.5,4,4.5,5];
tsink_data = [25.8,18.8,12.8,9.3,8.4,6.0,4.6];

%Water density in g/mm^3
rho = 1.00e-3;

%Strength of gravity in mm/s^2
g = 9.81*1e3;

%Cup parameters (distances in mm, time /s)
r = 20.7; %Using approximate volume of actual cup.
h = 56-2; %Note two washers have a thickness of about 2mm

%Cup + washer mass /g
m = 42;

%Hole diameters /mm for model
N = 100;
d = linspace( 1, 5, N );

%Fontsize for graphs
fsize = 18;

%

%Determine line of best fit for tsink = a/d^2
x = d_data.^(-2); y = tsink_data;
[yfit,xfit,rr,a,da,yupper,ylower,s] = bestfit(x,y);

%Calculate vena contracta coeffeicient from tsink = a/d^2
C = ( h - m/(rho*pi*r^2) )*(4*r^3)*sqrt( rho*pi/(2*m*g) )/a;

%Run models
for n=1:N
    [tsink(n), model(n).t, model(n).V, model(n).x,...
        model(n).z, model(n).a ] = cyl_sinking_model( d(n),C,rho,g,r,h,m );
end

%

%% Compare model to experimental data %%

%Plot tsink vs d
plot( d, tsink ); hold on; plot( d_data, tsink_data,'r+','markersize',fsize );
set(gca,'fontsize',fsize ); grid on;
legend({'Model','Experiment'},'fontsize',fsize,'location','northwest');
xlabel( 'Hole diameter d /mm');
ylabel( 'Sinking time /s' );
title('Election cup sinking time vs hole diameter');
print( gcf,'sinking time vs d.png','-r300','-dpng');
clf;

%Plot tsink vs 1/d^2
plot( d.^(-2), tsink); hold on;
plot( d_data.^(-2), tsink_data,'r+','markersize',fsize );
set(gca,'fontsize',fsize ); grid on;
legend({'Model','Experiment'},'fontsize',fsize,'location','northwest');
xlabel( '1/(Hole diameter d /mm)^2','fontsize',fsize );
ylabel( 'Sinking time /s','fontsize',fsize );
title([' t_{sink} = (',num2str(a,3),'+/-',num2str(da,3),')/d^2, C = ',num2str(C,3)],'fontsize',fsize );
print( gcf,'sinking time vs 1 over d squared.png','-r300','-dpng');
close(gcf);

%

%% Plot example sinking x,V,z vs t graphs %%
n = 1;
t = model(n).t; z = model(n).z; x = model(n).x; V = model(n).V;

% t vs z and x
plot( t, z, t, x); set(gca,'fontsize',fsize); grid on;
legend({'z','x'},'fontsize',fsize,'location','northeast');
xlabel('time t /s');
ylabel('Water height z and depth from lip x /mm');
title(['Sinking profile for d=',num2str(d(n)),...
    'mm. t_{sink}=',num2str(tsink(n)),'s.']);
print( gcf, ['Sinking profile for d=',strrep(num2str(d(n)),'.','p'),'mm.png'],...
    '-dpng','-r300');

%t vs V
plot( t, V/1000); set(gca,'fontsize',fsize); grid on;
xlabel('time t /s');
ylabel('Water volume /cm^3');
title(['Sinking profile for d=',num2str(d(n)),...
    'mm. t_{sink}=',num2str(tsink(n)),'s.']);
print( gcf, ['Sinking V profile for d=',strrep(num2str(d(n)),'.','p'),'mm.png'],...
    '-dpng','-r300');
close(gcf);

%%

%Cyclindrical sinking model (this is analytic!), as a function of hole
% diameter d /mm cup radius r /mm, cup height h /mm, cup mass m /g, and vena contracta C. 
% rho is fluid density in g/mm^3 and g is strength of gravity in mm/s^2
function [tsink, t, V, x, z, a ] = cyl_sinking_model(d,C,rho,g,r,h,m)

%Calculate sinking time /s
a =( h - m/(rho*pi*r^2) ) * ( 4*(r^3)/C ) * sqrt( (rho*pi)/(2*m*g) );
tsink = a/(d^2);

%Define time array /s
t = linspace( 0, tsink, 1000 );

%Define x,z,V   (distances in mm, volume in cubic mm
z = ( C*(d^2)/(4*r^3) )*sqrt( 2*m*g/(rho*pi) )*t;
x = h - z - m/(rho*pi*r^2);
V = pi*(r^2)*z;

%%


%Line of best fit function yfit = m*x, with product moment correlation
%coefficient r
function [yfit,xfit,r,m,dm,yupper,ylower,s] = bestfit(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = []; y(ignore) = [];
n = length(x);

%Compute line of best fit
xbar = mean(x); ybar = mean(y); xybar = mean(x.*y);
xxbar = mean(x.^2 ); yybar = mean(y.^2 );
Vx = xxbar - xbar^2; Vy = yybar - ybar^2;
COVxy = xybar - xbar*ybar;
m = xybar/xxbar; r = COVxy/sqrt( Vx*Vy );
[x,i] = sort(x); y = y(i);
yfit = m*x; xfit = x;

%Compute error in gradient m
s = sqrt( (1/(n-1))*sum( (y - yfit).^2 ) );
dm = s/sqrt(n*Vx);

%Determine envelope lines
yupper = (m+dm)*x;
ylower = (m-dm)*x;

%End of code